#procedure ExpandGam(maxsum)
****************************************************************************
**
**  Filename:
**            ExpandGam.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Expands \keyword{Gamma} and \keyword{InvGamma} functions 
**            in the small parameter \keyword{ep}, 
**            i.e. $\Gamma(i+a\epsilon)$ and 
**            $1/\Gamma(i+a\epsilon)$ in $\epsilon$, where $i$ and $a$ take 
**            integer values.
**            The argument specifies the number of the highest sum 
**            in the expression. 
**            We choose the \mbox{$\overline{\mbox{MS}}$-scheme}, i.e. 
**            $\exp(-\gamma_E a \epsilon) = 1$, where $\gamma_E$ is 
**            Euler's constant.
**
**  Comments:
**            epow counts the powers of epsilon (to avoid pushing things
**            over the edge (temporarily -> definitively gives errors))
**            We define
**           		G(a*ep) = Gamma(1+a*ep)
**          		InvG(a*ep) = 1/Gamma(1+a*ep)
**              and (assuming 0 < x 1 rational)
**              Gpq(x,a*ep) = Gamma(x+a*ep)
**              InvGpq(x,a*ep) = 1/Gamma(x+a*ep)
**
**  Revision: 
**            $Id: ExpandGam.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: ExpandGam.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
*csm added Fri Nov 25 16:35:16 CET 2005
#define SCHEME "MSbar"
*csm end
*
*
* (1) expansion of Gamma(j1+n+a*ep) with n integer, j1 symbolic
*
* GamNum(n,ep) = exp(-sum(k,1,inf,ep^k*sign(k)/k*S(R(k),n)))
* GamDen(n,ep) = exp(+sum(k,1,inf,ep^k*sign(k)/k*S(R(k),n)))
*
SplitArg,((ep)),Gamma,InvGamma;
id 	Gamma(x?,ep?) = Gammaa(x,ep);
id 	InvGamma(x?,ep?) = InvGammaa(x,ep);
id Gamma(x1?!number_,?a,ep?) = Gamma(0,x1,?a,ep);
id InvGamma(x1?!number_,?a,ep?) = InvGamma(0,x1,?a,ep);
repeat 	id Gamma(n?int_,x1?,x2?,?a,ep?) = Gamma(n,x1+x2,?a,ep);
repeat 	id InvGamma(n?int_,x1?,x2?,?a,ep?) = InvGamma(n,x1+x2,?a,ep);
id 	Gamma(n?int_,x?,ep?) = Gammaa(n+x,ep);
id 	InvGamma(n?int_,x?,ep?) = InvGammaa(n+x,ep);
repeat 	id Gamma(x1?,x2?,?a,ep?) = Gamma(x1+x2,?a,ep);
repeat 	id InvGamma(x1?,x2?,?a,ep?) = InvGamma(x1+x2,?a,ep);


#do counter = 1,1
id,once,Gammaa(n?,ep?) = 
	+theta_(n-1)*fac_(n-1)*G(ep)*GamNum(n-1,ep)
	+theta_(-n)*sign_(-n)*den(ep)*invfac_(-n)*G(ep)*GamDen(-n,-ep);
SplitArg,((ep)),num,den;
id 	den(0,x?) = dena(x/ep,x);
id	num(0,x?) = numa(x/ep,x);
id	dena(x?,ep?) = epow(-1)/x;
id	numa(x?,ep?) = epow(1)*x;
id	den(x?,ep?) = den(x+ep);
id	num(x?,ep?) = num(x+ep);
id	num(x?)*den(x?) = 1;
repeat 	id epow(n1?)*epow(n2?) = epow(n1+n2);
id	theta_(x?) = theta(x);
if 	( count(Gammaa,1) ) redefine counter "0";
#do i = `maxsum',1,-1
#call 	DoTheta(`i')
.sort(PolyFun=acc):Gamma->GamNum,GamDen `i';
#enddo
#call 	DoTheta(0)
.sort(PolyFun=acc):Gamma->GamNum,GamDen 0;
#enddo

#do counter = 1,1
id,once,InvGammaa(n?,ep?) = 
	+theta_(n-1)*invfac_(n-1)*InvG(ep)*GamDen(n-1,ep)
	+theta_(-n)*sign_(-n)*num(ep)*fac_(-n)*InvG(ep)*GamNum(-n,-ep);
SplitArg,((ep)),num,den;
id 	den(0,x?) = dena(x/ep,x);
id	num(0,x?) = numa(x/ep,x);
id	dena(x?,ep?) = epow(-1)/x;
id	numa(x?,ep?) = epow(1)*x;
id	den(x?,ep?) = den(x+ep);
id	num(x?,ep?) = num(x+ep);
id	num(x?)*den(x?) = 1;
repeat 	id epow(n1?)*epow(n2?) = epow(n1+n2);
id	theta_(x?) = theta(x);
if 	( count(InvGammaa,1) ) redefine counter "0";
#do i = `maxsum',1,-1
#call 	DoTheta(`i')
.sort(PolyFun=acc):InvGam->GamNum,GamDen `i';
#enddo
#call 	DoTheta(0)
.sort(PolyFun=acc):InvGam->GamNum,GamDen 0;
#enddo

repeat 	id epow(x1?)*epow(x2?) = epow(x1+x2);
id	G(x?)*InvG(x?) = 1;
.sort(PolyFun=acc):GamNum+GamDen-int;

*
*
* (2) expansion of Gamma(j1+n+a*ep) with p,q integer, j1 symbolic
*
* GamNumpq(x,n,ep) = exp(-sum(k,1,inf,ep^k*sign(k)/k*sum(i,1,n,den(i+x)^k)))
* GamDenpq(x,n,ep) = exp(+sum(k,1,inf,ep^k*sign(k)/k*sum(i,1,n,den(i+x)^k)))
*
SplitArg,((ep)),Gamma,InvGamma;
repeat 	id Gamma(n?number_,x1?,x2?,?a,ep?) = Gamma(n,x1+x2,?a,ep);
repeat 	id InvGamma(n?number_,x1?,x2?,?a,ep?) = InvGamma(n,x1+x2,?a,ep);
id 	Gamma(n?number_,x?,ep?) = Gammaa(n-integer_(n),integer_(n)+x,ep);
id 	InvGamma(n?number_,x?,ep?) = InvGammaa(n-integer_(n),integer_(n)+x,ep);

#do counter = 1,1
id,once,Gammaa(x?number_,n?,ep?) = 
	+theta_(n-1)*num(x+ep)*Gpq(x,ep)*Gamma(n+x)*InvGamma(1+x)*GamNumpq(x,n-1,ep)
	+theta_(-n)*Gpq(x,ep)*Gamma(1-x)*InvGamma(-n+1-x)*sign_(-n)*GamDenpq(-x,-n,-ep);
SplitArg,((ep)),num,den;
id 	den(0,x?) = dena(x/ep,x);
id	num(0,x?) = numa(x/ep,x);
id	dena(x?,ep?) = epow(-1)/x;
id	numa(x?,ep?) = epow(1)*x;
id	den(x?,ep?) = den(x+ep);
id	num(x?,ep?) = num(x+ep);
id	num(x?)*den(x?) = 1;

repeat 	id epow(n1?)*epow(n2?) = epow(n1+n2);
id	theta_(x?) = theta(x);

#do i = `maxsum',1,-1
#call 	DoTheta(`i')
.sort(PolyFun=acc):Gamma->GamNum,GamDen `i';
#enddo
#call 	DoTheta(0)
.sort(PolyFun=acc):Gamma->GamNum,GamDen 0;
#enddo

#do counter = 1,1
id,once,InvGammaa(x?number_,n?,ep?) = 
	+theta_(n-1)*den(x+ep)*InvGpq(x,ep)*Gamma(1+x)*InvGamma(n+x)*GamDenpq(x,n-1,ep)
	+theta_(-n)*InvGpq(x,ep)*Gamma(-n+1-x)*InvGamma(1-x)*sign_(-n)*GamNumpq(-x,-n,-ep);
SplitArg,((ep)),num,den;
id 	den(0,x?) = dena(x/ep,x);
id	num(0,x?) = numa(x/ep,x);
id	dena(x?,ep?) = epow(-1)/x;
id	numa(x?,ep?) = epow(1)*x;
id	den(x?,ep?) = den(x+ep);
id	num(x?,ep?) = num(x+ep);
id	num(x?)*den(x?) = 1;

repeat 	id epow(n1?)*epow(n2?) = epow(n1+n2);
id	theta_(x?) = theta(x);
if 	( count(InvGammaa,1) ) redefine counter "0";
#do i = `maxsum',1,-1
#call 	DoTheta(`i')
.sort(PolyFun=acc):InvGam->GamNum,GamDen `i';
#enddo
#call 	DoTheta(0)
.sort(PolyFun=acc):InvGam->GamNum,GamDen 0;
#enddo

repeat 	id epow(x1?)*epow(x2?) = epow(x1+x2);
id	Gpq(n?,x?)*InvGpq(n?,x?) = 1;
.sort(PolyFun=acc):GamNum+GamDen-int;

*
* look for simplifications among left-over factorials and Gamma functions
*
#call SimplifyFacGam

.sort(PolyFun=acc):Simplify fac/Gam;

*
* expand den(n+m*ep) in ep with n being a non-symbolic integer 
* 
#call ExpandDen(0)
id	epow(n?int_) = acc(ep^n);

*
*	Finally we expand the GamNum and GamDen functions; generation of S-sums
*
* GamNum(n,ep) = exp(-sum(k,1,inf,ep^k*sign(k)/k*S(R(k),n)))
* GamDen(n,ep) = exp(+sum(k,1,inf,ep^k*sign(k)/k*S(R(k),n)))
*
* GamNumpq(x,n,ep) = exp(-sum(k,1,inf,ep^k*sign(k)/k*sum(i,1,n,den(i+x)^k)))
* GamDenpq(x,n,ep) = exp(+sum(k,1,inf,ep^k*sign(k)/k*sum(i,1,n,den(i+x)^k)))
*

id	acc(x?) = x;
repeat;
	id,once,GamDen(n?,x?)*ep^a? =
				sum_(jjj,0,`EXPANDEP'-a,sign_(jjj)*x^jjj*xx^jjj*R(R,X,n))*ep^a + ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
	repeat id,R(R(?a),X(?x),n?)*xx = R(R(1,?a),X(1,?x),n);
	id	R(R,X,n?) = 1;
	id	R(R(?a),X(?x),0) = 0;
	id	R(R(?a),X(?x),1) = 1;
	id	R(R(?a),X(?x),n?) = S(R(?a),X(?x),n);
endrepeat;
.sort(PolyFun): GamDen-step;

id	acc(x?) = x;
repeat;
	id,once,GamNum(n?,x?)*ep^a? =
				sum_(jjj,0,`EXPANDEP'-a,x^jjj*xx^jjj*R(R,X,n))*ep^a + ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
	id	R(R,X,n?)*xx = R(R(1),X(1),n);
	id	R(R,X,n?) = 1;
	id	R(R(?a),X(?x),0) = 0;
	repeat id,R(R(x1?,?a),X(?x),n?)*xx = R(R(1,x1,?a),X(1,?x),n)-R(R(x1+1,?a),X(?x),n);
	id	R(R(?a),X(?x),1) = 1;
	id	R(R(?a),X(?x),n?) = S(R(?a),X(?x),n);
endrepeat;

id  GamNum(0,x?) = 1;
id  GamDen(0,x?) = 1;

*csm added Fri Nov 25 16:35:16 CET 2005
#if(`SCHEME' == MSbar) 
* adopt MSbar scheme
	#define 	lowerbound "2"
#else
	#define 	lowerbound "1"
#endif
*csm end
.sort(PolyFun=acc):ExpandGam_fin;
#call BasisS(0)
#call SubsS(1)

*
* expand Gamma(1+n*ep) 
*
#do counter = 1,1
if(count(G,1,InvG,1)) id	acc(x?) = x;
#do icounter = 1,`MAXCOUNTER'
id,once, G(`icounter'*ep)*ep^a? = (
	1+
	sum_(jj,1,`EXPANDEP'-a,invfac_(jj)*(sum_(jjj,`lowerbound',`EXPANDEP'-a-jj+1,zriemann(jjj)/jjj*(-`icounter'*ep)^jjj))^jj)
		)*ep^a
	+ ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
id,once, G(-`icounter'*ep)*ep^a? = (
	1+
	sum_(jj,1,`EXPANDEP'-a,invfac_(jj)*(sum_(jjj,`lowerbound',`EXPANDEP'-a-jj+1,zriemann(jjj)/jjj*(+`icounter'*ep)^jjj))^jj)
		)*ep^a
	+ ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
id,once, InvG(`icounter'*ep)*ep^a? = (
	1+
	sum_(jj,1,`EXPANDEP'-a,sign_(jj)*invfac_(jj)*(sum_(jjj,`lowerbound',`EXPANDEP'-a-jj+1,zriemann(jjj)/jjj*(-`icounter'*ep)^jjj))^jj)
		)*ep^a
	+ ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
id,once, InvG(-`icounter'*ep)*ep^a? = (
	1+
	sum_(jj,1,`EXPANDEP'-a,sign_(jj)*invfac_(jj)*(sum_(jjj,`lowerbound',`EXPANDEP'-a-jj+1,zriemann(jjj)/jjj*(+`icounter'*ep)^jjj))^jj)
		)*ep^a
	+ ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
repeat id zriemann(`icounter') = z`icounter';
#enddo
if(count(G,1,InvG,1)) Redefine counter "0"; 
.sort(PolyFun=acc):G(n*ep);
#enddo

*csm added Fri Nov 25 16:35:16 CET 2005
#do icounter = 1,`MAXCOUNTER'
repeat id zriemann(`icounter') = z`icounter';
#enddo
*csm end
repeat, id order({`EXPANDEP'+1})*order({`EXPANDEP'+1}) = order({`EXPANDEP'+1});

#endprocedure
*
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
